@@ -1,21 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "../examples/common/utility/utility.h"
@@ -38,7 +34,7 @@
 #include <algorithm>
 #include "tbb/mutex.h"
 
-//TODO: probably move this to separate header utlity file
+//TODO: probably move this to separate header utility file
 namespace micro_benchmarking{
 namespace utils{
     template <typename type>
@@ -54,11 +50,12 @@ namespace utils{
     // this at beginning of program; sets one_us_iters to number of iters to
     // busy_wait for approx. 1 us
     void calibrate_busy_wait() {
+        const unsigned niter = 1000000;
         tbb::tick_count t0 = tbb::tick_count::now();
-        for (volatile unsigned int i=0; i<1000000; ++i) continue;
+        for (volatile unsigned int i=0; i<niter; ++i) continue;
         tbb::tick_count t1 = tbb::tick_count::now();
 
-        one_us_iters = (unsigned int)((1000000.0/(t1-t0).seconds())*0.000001);
+        one_us_iters = (unsigned int)(niter/(t1-t0).seconds())*1e-6;
     }
 
     void busy_wait(int us)
@@ -161,7 +158,7 @@ struct throughput {
                         (std::min)(p.time_check_granularity_ops, _this->per_thread_sample_size - part_of_the_sample_so_far);
 
                 for (access_sequence_type::iterator it = iteration_begin_it; it < iteration_end_it; ++it){
-                    typename cache_type::handle h = _this->m_cache(*it);
+                    typename cache_type::handle h = _this->m_cache[*it];
                     micro_benchmarking::utils::busy_wait(p.time_of_item_use_usec);
                     micro_benchmarking::utils::disable_elimination(h.value());
                 }
