@@ -1,25 +1,22 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "harness.h"
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+#if __TBB_CPF_BUILD
+#define TBB_DEPRECATED_FLOW_NODE_EXTRACTION 1
 #include "harness_graph.h"
 #endif
 #include "tbb/flow_graph.h"
@@ -29,65 +26,86 @@
 const int L = 10;
 const int N = 1000;
 
+using tbb::flow::internal::SUCCESSFULLY_ENQUEUED;
+
 template< typename T >
-struct serial_receiver : public tbb::flow::receiver<T> {
+struct serial_receiver : public tbb::flow::receiver<T>, NoAssign {
    T next_value;
+   tbb::flow::graph& my_graph;
 
-   serial_receiver() : next_value(T(0)) {}
+   serial_receiver(tbb::flow::graph& g) : next_value(T(0)), my_graph(g) {}
 
-   /* override */ tbb::task *try_put_task( const T &v ) {
+   tbb::task *try_put_task( const T &v ) __TBB_override {
        ASSERT( next_value++  == v, NULL );
-       return const_cast<tbb::task *>(tbb::flow::interface7::SUCCESSFULLY_ENQUEUED);
+       return const_cast<tbb::task *>(SUCCESSFULLY_ENQUEUED);
    }
 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
-    void internal_add_built_predecessor( tbb::flow::sender<T> & ) { }
-    void internal_delete_built_predecessor( tbb::flow::sender<T> & ) { }
-    void copy_predecessors( std::vector<tbb::flow::sender<T>*> & ) { }
-    size_t predecessor_count() { return 0; }
+    tbb::flow::graph& graph_reference() __TBB_override {
+        return my_graph;
+    }
+
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
+    typedef typename tbb::flow::receiver<T>::built_predecessors_type built_predecessors_type;
+    typedef typename tbb::flow::receiver<T>::predecessor_list_type predecessor_list_type;
+    typedef typename tbb::flow::receiver<T>::predecessor_type predecessor_type;
+    built_predecessors_type bpt;
+    built_predecessors_type &built_predecessors() __TBB_override { return bpt; }
+    void internal_add_built_predecessor( predecessor_type & ) __TBB_override { }
+    void internal_delete_built_predecessor( predecessor_type & ) __TBB_override { }
+    void copy_predecessors( predecessor_list_type & ) __TBB_override { }
+    size_t predecessor_count() __TBB_override { return 0; }
 #endif
 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
-   /*override*/void reset_receiver(tbb::flow::reset_flags /*f*/) {next_value = T(0);}
-#else
-   /*override*/void reset_receiver() {next_value = T(0);}
-#endif
+   void reset_receiver(tbb::flow::reset_flags /*f*/) __TBB_override {next_value = T(0);}
 };
 
 template< typename T >
-struct parallel_receiver : public tbb::flow::receiver<T> {
+struct parallel_receiver : public tbb::flow::receiver<T>, NoAssign {
 
-   tbb::atomic<int> my_count;
+    tbb::atomic<int> my_count;
+    tbb::flow::graph& my_graph;
 
-   parallel_receiver() { my_count = 0; }
+    parallel_receiver(tbb::flow::graph& g) : my_graph(g) { my_count = 0; }
 
-   /* override */ tbb::task *try_put_task( const T &/*v*/ ) {
+    tbb::task *try_put_task( const T &/*v*/ ) __TBB_override {
        ++my_count;
-       return const_cast<tbb::task *>(tbb::flow::interface7::SUCCESSFULLY_ENQUEUED);
-   }
+       return const_cast<tbb::task *>(tbb::flow::internal::SUCCESSFULLY_ENQUEUED);
+    }
 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
-    void internal_add_built_predecessor( tbb::flow::sender<T> & ) { }
-    void internal_delete_built_predecessor( tbb::flow::sender<T> & ) { }
-    void copy_predecessors( std::vector<tbb::flow::sender<T>*> & ) { }
-    size_t predecessor_count( ) { return 0; }
-   /*override*/void reset_receiver(tbb::flow::reset_flags /*f*/) {my_count = 0;}
-#else
-   /*override*/void reset_receiver() {my_count = 0;}
+    tbb::flow::graph& graph_reference() __TBB_override {
+        return my_graph;
+    }
+
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
+    typedef typename tbb::flow::receiver<T>::built_predecessors_type built_predecessors_type;
+    typedef typename tbb::flow::receiver<T>::predecessor_list_type predecessor_list_type;
+    typedef typename tbb::flow::receiver<T>::predecessor_type predecessor_type;
+    built_predecessors_type bpt;
+    built_predecessors_type &built_predecessors() __TBB_override { return bpt; }
+    void internal_add_built_predecessor( predecessor_type & ) __TBB_override { }
+    void internal_delete_built_predecessor( predecessor_type & ) __TBB_override { }
+    void copy_predecessors( predecessor_list_type & ) __TBB_override { }
+    size_t predecessor_count( ) __TBB_override { return 0; }
 #endif
+    void reset_receiver(tbb::flow::reset_flags /*f*/) __TBB_override {my_count = 0;}
 };
 
 template< typename T >
 struct empty_sender : public tbb::flow::sender<T> {
-        /* override */ bool register_successor( tbb::flow::receiver<T> & ) { return false; }
-        /* override */ bool remove_successor( tbb::flow::receiver<T> & ) { return false; }
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
-        void    internal_add_built_successor( tbb::flow::receiver<T> & ) { }
-        void internal_delete_built_successor( tbb::flow::receiver<T> & ) { }
-        void copy_successors( std::vector<tbb::flow::receiver<T>*> & ) { }
-        size_t successor_count() { return 0; }
-#endif
+        typedef typename tbb::flow::sender<T>::successor_type successor_type;
 
+        bool register_successor( successor_type & ) __TBB_override { return false; }
+        bool remove_successor( successor_type & ) __TBB_override { return false; }
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
+        typedef typename tbb::flow::sender<T>::built_successors_type built_successors_type;
+        typedef typename tbb::flow::sender<T>::successor_list_type successor_list_type;
+        built_successors_type bst;
+        built_successors_type &built_successors() __TBB_override { return bst; }
+        void    internal_add_built_successor( successor_type & ) __TBB_override { }
+        void internal_delete_built_successor( successor_type & ) __TBB_override { }
+        void copy_successors( successor_list_type & ) __TBB_override { }
+        size_t successor_count() __TBB_override { return 0; }
+#endif
 };
 
 
@@ -97,14 +115,14 @@ struct put_body : NoAssign {
     tbb::flow::limiter_node<T> &my_lim;
     tbb::atomic<int> &my_accept_count;
 
-    put_body( tbb::flow::limiter_node<T> &lim, tbb::atomic<int> &accept_count ) : 
+    put_body( tbb::flow::limiter_node<T> &lim, tbb::atomic<int> &accept_count ) :
         my_lim(lim), my_accept_count(accept_count) {}
 
     void operator()( int ) const {
         for ( int i = 0; i < L; ++i ) {
             bool msg = my_lim.try_put( T(i) );
             if ( msg == true )
-               ++my_accept_count; 
+               ++my_accept_count;
         }
     }
 };
@@ -115,7 +133,7 @@ struct put_dec_body : NoAssign {
     tbb::flow::limiter_node<T> &my_lim;
     tbb::atomic<int> &my_accept_count;
 
-    put_dec_body( tbb::flow::limiter_node<T> &lim, tbb::atomic<int> &accept_count ) : 
+    put_dec_body( tbb::flow::limiter_node<T> &lim, tbb::atomic<int> &accept_count ) :
         my_lim(lim), my_accept_count(accept_count) {}
 
     void operator()( int ) const {
@@ -126,25 +144,26 @@ struct put_dec_body : NoAssign {
                 ++local_accept_count;
                 ++my_accept_count;
                 my_lim.decrement.try_put( tbb::flow::continue_msg() );
-            } 
+            }
         }
     }
 
 };
 
 template< typename T >
-void test_puts_with_decrements( int num_threads, tbb::flow::limiter_node< T >& lim ) {
-    parallel_receiver<T> r;
+void test_puts_with_decrements( int num_threads, tbb::flow::limiter_node< T >& lim , tbb::flow::graph& g) {
+    parallel_receiver<T> r(g);
     empty_sender< tbb::flow::continue_msg > s;
     tbb::atomic<int> accept_count;
     accept_count = 0;
     tbb::flow::make_edge( lim, r );
     tbb::flow::make_edge(s, lim.decrement);
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
     ASSERT(lim.decrement.predecessor_count() == 1, NULL);
     ASSERT(lim.successor_count() == 1, NULL);
     ASSERT(lim.predecessor_count() == 0, NULL);
-    std::vector<tbb::flow::sender<tbb::flow::continue_msg> *> dec_preds;
+    typename tbb::flow::interface10::internal::decrementer
+        <tbb::flow::limiter_node<T>, tbb::flow::continue_msg>::predecessor_list_type dec_preds;
     lim.decrement.copy_predecessors(dec_preds);
     ASSERT(dec_preds.size() == 1, NULL);
 #endif
@@ -159,17 +178,17 @@ void test_puts_with_decrements( int num_threads, tbb::flow::limiter_node< T >& l
 // Tests
 //
 // limiter only forwards below the limit, multiple parallel senders / single receiver
-// mutiple parallel senders that put to decrement at each accept, limiter accepts new messages
-// 
-// 
+// multiple parallel senders that put to decrement at each accept, limiter accepts new messages
+//
+//
 template< typename T >
 int test_parallel(int num_threads) {
- 
+
    // test puts with no decrements
    for ( int i = 0; i < L; ++i ) {
        tbb::flow::graph g;
        tbb::flow::limiter_node< T > lim(g, i);
-       parallel_receiver<T> r;
+       parallel_receiver<T> r(g);
        tbb::atomic<int> accept_count;
        accept_count = 0;
        tbb::flow::make_edge( lim, r );
@@ -184,9 +203,9 @@ int test_parallel(int num_threads) {
    for ( int i = 1; i < L; ++i ) {
        tbb::flow::graph g;
        tbb::flow::limiter_node< T > lim(g, i);
-       test_puts_with_decrements(num_threads, lim);
+       test_puts_with_decrements(num_threads, lim, g);
        tbb::flow::limiter_node< T > lim_copy( lim );
-       test_puts_with_decrements(num_threads, lim_copy);
+       test_puts_with_decrements(num_threads, lim_copy, g);
    }
 
    return 0;
@@ -197,15 +216,15 @@ int test_parallel(int num_threads) {
 //
 // limiter only forwards below the limit, single sender / single receiver
 // at reject, a put to decrement, will cause next message to be accepted
-// 
+//
 template< typename T >
 int test_serial() {
- 
+
    // test puts with no decrements
    for ( int i = 0; i < L; ++i ) {
        tbb::flow::graph g;
        tbb::flow::limiter_node< T > lim(g, i);
-       serial_receiver<T> r;
+       serial_receiver<T> r(g);
        tbb::flow::make_edge( lim, r );
        for ( int j = 0; j < L; ++j ) {
            bool msg = lim.try_put( T(j) );
@@ -218,7 +237,7 @@ int test_serial() {
    for ( int i = 1; i < L; ++i ) {
        tbb::flow::graph g;
        tbb::flow::limiter_node< T > lim(g, i);
-       serial_receiver<T> r;
+       serial_receiver<T> r(g);
        empty_sender< tbb::flow::continue_msg > s;
        tbb::flow::make_edge( lim, r );
        tbb::flow::make_edge(s, lim.decrement);
@@ -302,13 +321,13 @@ test_multifunction_to_limiter(int _max, int _nparallel) {
     tbb::flow::make_edge(tbb::flow::output_port<DECREMENT_OUTPUT>(mf_node), lim_node.decrement);
     tbb::flow::make_edge(lim_node, fn_node);
     tbb::flow::make_edge(fn_node, mf_node);
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
     REMARK("pred cnt == %d\n",(int)(lim_node.predecessor_count()));
     REMARK("succ cnt == %d\n",(int)(lim_node.successor_count()));
-    tbb::flow::limiter_node<int>::successor_vector_type my_succs;
+    tbb::flow::limiter_node<int>::successor_list_type my_succs;
     lim_node.copy_successors(my_succs);
     REMARK("succ cnt from vector  == %d\n",(int)(my_succs.size()));
-    tbb::flow::limiter_node<int>::predecessor_vector_type my_preds;
+    tbb::flow::limiter_node<int>::predecessor_list_type my_preds;
     lim_node.copy_predecessors(my_preds);
     REMARK("pred cnt from vector  == %d\n",(int)(my_preds.size()));
 #endif
@@ -344,26 +363,62 @@ test_continue_msg_reception() {
     ASSERT(qn.try_get(outint) && outint == 42, "initial put to decrement stops node");
 }
 
+#if TBB_DEPRECATED_LIMITER_NODE_CONSTRUCTOR
+using namespace tbb::flow;
+void run_and_check_result(graph& g, limiter_node<int>& limit, queue_node<int>& queue, broadcast_node<continue_msg>& broad) {
+    ASSERT( limit.try_put(1), NULL );
+    ASSERT( limit.try_put(2), NULL );
+    ASSERT( !limit.try_put(3), NULL );
+    ASSERT( broad.try_put(continue_msg()), NULL );
+    ASSERT( limit.decrement.try_put(continue_msg()), NULL );
+    ASSERT( limit.try_put(4), NULL );
+    ASSERT( !limit.try_put(5), NULL );
+    g.wait_for_all();
 
+    int list[] = {1, 2, 4};
+    int var = 0;
+    for (size_t i = 0; i < sizeof(list)/sizeof(list[0]); i++) {
+        queue.try_get(var);
+        ASSERT(var==list[i], "some data dropped, input does not match output");
+    }
+}
+
+void test_num_decrement_predecessors() {
+    graph g;
+    queue_node<int> output_queue(g);
+    limiter_node<int> limit1(g, 2, /*number_of_predecessors*/1);
+    limiter_node<int, continue_msg> limit2(g, 2, /*number_of_predecessors*/1);
+    broadcast_node<continue_msg> broadcast(g);
+
+    make_edge(limit1, output_queue);
+    make_edge(limit2, output_queue);
+
+    make_edge(broadcast, limit1.decrement);
+    make_edge(broadcast, limit2.decrement);
+
+    run_and_check_result(g, limit1, output_queue, broadcast);
+    run_and_check_result(g, limit2, output_queue, broadcast);
+}
+#else // TBB_DEPRECATED_LIMITER_NODE_CONSTRUCTOR
 //
 // This test ascertains that if a message is not successfully put
 // to a successor, the message is not dropped but released.
 //
 
-using namespace tbb::flow;
 void test_reserve_release_messages() {
+    using namespace tbb::flow;
     graph g;
 
     //making two queue_nodes: one broadcast_node and one limiter_node
     queue_node<int> input_queue(g);
     queue_node<int> output_queue(g);
-    broadcast_node<continue_msg> broad(g);
-    limiter_node<int> limit(g,2,1); //threshold of 2
+    broadcast_node<int> broad(g);
+    limiter_node<int, int> limit(g,2); //threshold of 2
 
     //edges
     make_edge(input_queue, limit);
     make_edge(limit, output_queue);
-    make_edge(broad,limit.decrement); 
+    make_edge(broad,limit.decrement);
 
     int list[4] = {19, 33, 72, 98}; //list to be put to the input queue
 
@@ -374,29 +429,71 @@ void test_reserve_release_messages() {
 
     remove_edge(limit, output_queue); //remove successor
 
-    //sending continue messages to the decrement port of the limiter
-    broad.try_put(continue_msg());
-    broad.try_put(continue_msg()); //failed message retrieved.
+    //sending message to the decrement port of the limiter
+    broad.try_put(1); //failed message retrieved.
     g.wait_for_all();
 
     make_edge(limit, output_queue); //putting the successor back
 
-    broad.try_put(continue_msg());
-    broad.try_put(continue_msg());  //drop the count
+    broad.try_put(1);  //drop the count
 
     input_queue.try_put(list[3]);  //success
     g.wait_for_all();
 
     int var=0;
 
-    for (int i=0; i<4; i++){
-    output_queue.try_get(var);
-    ASSERT(var==list[i], "some data dropped, input does not match output");
-    g.wait_for_all();
-  }
+    for (int i=0; i<4; i++) {
+        output_queue.try_get(var);
+        ASSERT(var==list[i], "some data dropped, input does not match output");
+        g.wait_for_all();
+    }
 }
 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+void test_decrementer() {
+    const int threshold = 5;
+    tbb::flow::graph g;
+    tbb::flow::limiter_node<int, int> limit(g, threshold);
+    tbb::flow::queue_node<int> queue(g);
+    make_edge(limit, queue);
+    int m = 0;
+    ASSERT( limit.try_put( m++ ), "Newly constructed limiter node does not accept message." );
+    ASSERT( limit.decrement.try_put( -threshold ), // close limiter's gate
+            "Limiter node decrementer's port does not accept message." );
+    ASSERT( !limit.try_put( m++ ), "Closed limiter node's accepts message." );
+    ASSERT( limit.decrement.try_put( threshold + 5 ),  // open limiter's gate
+            "Limiter node decrementer's port does not accept message." );
+    for( int i = 0; i < threshold; ++i )
+        ASSERT( limit.try_put( m++ ), "Limiter node does not accept message while open." );
+    ASSERT( !limit.try_put( m ), "Limiter node's gate is not closed." );
+    g.wait_for_all();
+    int expected[] = {0, 2, 3, 4, 5, 6};
+    int actual = -1; m = 0;
+    while( queue.try_get(actual) )
+        ASSERT( actual == expected[m++], NULL );
+    ASSERT( sizeof(expected) / sizeof(expected[0]) == m, "Not all messages have been processed." );
+    g.wait_for_all();
+
+    const size_t threshold2 = size_t(-1);
+    tbb::flow::limiter_node<int, long long> limit2(g, threshold2);
+    make_edge(limit2, queue);
+    ASSERT( limit2.try_put( 1 ), "Newly constructed limiter node does not accept message." );
+    long long decrement_value = (long long)( size_t(-1)/2 );
+    ASSERT( limit2.decrement.try_put( -decrement_value ),
+            "Limiter node decrementer's port does not accept message" );
+    ASSERT( limit2.try_put( 2 ), "Limiter's gate should not be closed yet." );
+    ASSERT( limit2.decrement.try_put( -decrement_value ),
+            "Limiter node decrementer's port does not accept message" );
+    ASSERT( !limit2.try_put( 3 ), "Overflow happened for internal counter." );
+    int expected2[] = {1, 2};
+    actual = -1; m = 0;
+    while( queue.try_get(actual) )
+        ASSERT( actual == expected2[m++], NULL );
+    ASSERT( sizeof(expected2) / sizeof(expected2[0]) == m, "Not all messages have been processed." );
+    g.wait_for_all();
+}
+#endif // TBB_DEPRECATED_LIMITER_NODE_CONSTRUCTOR
+
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
 void test_extract() {
     tbb::flow::graph g;
     int j;
@@ -408,7 +505,7 @@ void test_extract() {
     tbb::flow::broadcast_node<tbb::flow::continue_msg> b1(g);
 
     for( int i = 0; i < 2; ++i ) {
-
+        REMARK("At pass %d\n", i);
         ASSERT(node0.predecessor_count() == 0, "incorrect predecessor count at start");
         ASSERT(node0.successor_count() == 0, "incorrect successor count at start");
         ASSERT(node0.decrement.predecessor_count() == 0, "incorrect decrement pred count at start");
@@ -433,7 +530,8 @@ void test_extract() {
         ASSERT(node0.predecessor_count() == 2, "incorrect predecessor count after construction");
         ASSERT(node0.successor_count() == 1, "incorrect successor count after construction");
         ASSERT(node0.decrement.predecessor_count() == 2, "incorrect decrement pred count after construction");
-        ASSERT(q2.try_get(j) && j == i, "improper value forwarded to output queue");
+        ASSERT(q2.try_get(j), "fetch of value forwarded to output queue failed");
+        ASSERT(j == i, "improper value forwarded to output queue");
         q0.try_put(2*i);
         g.wait_for_all();
         ASSERT(!q2.try_get(j), "limiter_node forwarded item improperly");
@@ -444,12 +542,12 @@ void test_extract() {
         g.wait_for_all();
         ASSERT(q2.try_get(j) && j == 2*i, "limiter_node failed to forward item");
 
-        tbb::flow::limiter_node<int>::successor_vector_type sv;
-        tbb::flow::limiter_node<int>::predecessor_vector_type pv;
-        tbb::flow::continue_receiver::predecessor_vector_type dv;
-        std::vector<tbb::flow::receiver<int>*> sv1;
-        std::vector<tbb::flow::sender<int>*> pv1;
-        std::vector<tbb::flow::sender<tbb::flow::continue_msg>*> dv1;
+        tbb::flow::limiter_node<int>::successor_list_type sv;
+        tbb::flow::limiter_node<int>::predecessor_list_type pv;
+        tbb::flow::continue_receiver::predecessor_list_type dv;
+        tbb::flow::limiter_node<int>::successor_list_type sv1;
+        tbb::flow::limiter_node<int>::predecessor_list_type pv1;
+        tbb::flow::continue_receiver::predecessor_list_type dv1;
 
         node0.copy_predecessors(pv);
         node0.copy_successors(sv);
@@ -462,12 +560,15 @@ void test_extract() {
 
         ASSERT(pv.size() == 2, "improper size for predecessors");
         ASSERT(sv.size() == 1, "improper size for successors");
-        ASSERT(lists_match(pv,pv1), "predecesosr lists do not match");
+        ASSERT(lists_match(pv,pv1), "predecessor lists do not match");
         ASSERT(lists_match(sv,sv1), "successor lists do not match");
         ASSERT(lists_match(dv,dv1), "successor lists do not match");
 
         if(i == 0) {
             node0.extract();
+            ASSERT(node0.predecessor_count() == 0, "incorrect predecessor count after extraction");
+            ASSERT(node0.successor_count() == 0, "incorrect successor count after extraction");
+            ASSERT(node0.decrement.predecessor_count() == 0, "incorrect decrement pred count after extraction");
         }
         else {
             q0.extract();
@@ -487,7 +588,7 @@ void test_extract() {
             pv1.push_back(&(q1));
             dv1.push_back(&(b1));
 
-            ASSERT(lists_match(pv,pv1), "predecesosr lists do not match second iter");
+            ASSERT(lists_match(pv,pv1), "predecessor lists do not match second iter");
             ASSERT(lists_match(sv,sv1), "successor lists do not match second iter");
             ASSERT(lists_match(dv,dv1), "successor lists do not match second iter");
 
@@ -501,9 +602,9 @@ void test_extract() {
     }
 
 }
-#endif  // TBB_PREVIEW_FLOW_GRAPH_FEATURES
+#endif  // TBB_DEPRECATED_FLOW_NODE_EXTRACTION
 
-int TestMain() { 
+int TestMain() {
     for (int i = 1; i <= 8; ++i) {
         tbb::task_scheduler_init init(i);
         test_serial<int>();
@@ -513,8 +614,13 @@ int TestMain() {
     test_multifunction_to_limiter(30,3);
     test_multifunction_to_limiter(300,13);
     test_multifunction_to_limiter(3000,1);
+#if TBB_DEPRECATED_LIMITER_NODE_CONSTRUCTOR
+    test_num_decrement_predecessors();
+#else
     test_reserve_release_messages();
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+    test_decrementer();
+#endif
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
     test_extract();
 #endif
    return Harness::Done;
