@@ -1,21 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "tbb/scalable_allocator.h"
@@ -141,7 +137,7 @@ extern "C" void threadDtor(void*) {
 
 inline bool intersectingObjects(const void *p1, const void *p2, size_t n)
 {
-    return (size_t)labs((uintptr_t)p1 - (uintptr_t)p2) < n;
+    return p1>p2 ? ((uintptr_t)p1-(uintptr_t)p2)<n : ((uintptr_t)p2-(uintptr_t)p1)<n;
 }
 
 struct TestThread: NoAssign {
@@ -162,8 +158,12 @@ struct TestThread: NoAssign {
 
 // test releasing memory from pthread key destructor
 void TestKeyDtor() {
+    // Allocate region for large objects to prevent whole region release
+    // on scalable_free(currLarge) call, which result in wrong assert inside intersectingObjects check
+    void* preventLargeRelease = scalable_malloc(32*1024);
     for (int i=0; i<4; i++)
         NativeParallelFor( 1, TestThread(1) );
+    scalable_free(preventLargeRelease);
 }
 
 #endif // _WIN32||_WIN64
