@@ -1,21 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "tbb/parallel_do.h"
@@ -23,11 +19,10 @@
 #include "tbb/atomic.h"
 #include "harness.h"
 #include "harness_cpu.h"
-#include <deque>
 
 #if defined(_MSC_VER) && defined(_Wp64)
     // Workaround for overzealous compiler warnings in /Wp64 mode
-    #pragma warning (disable: 4267)
+    // #pragma warning (disable: 4267)
 #endif /* _MSC_VER && _Wp64 */
 
 #define N_DEPTHS     20
@@ -39,7 +34,10 @@ class value_t {
     value_t& operator= ( const value_t& );
 public:
     value_t ( size_t xx ) : x(xx) { ++g_values_counter; }
-    value_t ( const value_t& v ) : x(v.value()) { ++g_values_counter; }
+    value_t ( const value_t& v ) : x(v.x) { ++g_values_counter; }
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+    value_t ( value_t&& v ) : x(v.x) { ++g_values_counter; }
+#endif
     ~value_t () { --g_values_counter; }
     size_t value() const volatile { return x; }
 };
@@ -89,12 +87,23 @@ public:
     }
 };
 
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+/** Work item is passed by rvalue reference here. **/
+class FakeTaskGeneratorBody_RvalueRefVersion {
+public:
+    void operator() ( value_t&& depth ) const {
+        g_tasks_observed += FindNumOfTasks(depth.value());
+    }
+};
+#endif
+
 void do_work ( const value_t& depth, tbb::parallel_do_feeder<value_t>& feeder ) {
     ++g_tasks_observed;
-    size_t  d=depth.value();
-    --d;
-    for( size_t i = 0; i < depth.value(); ++i)
-        feeder.add(value_t(d));
+    value_t new_value(depth.value()-1);
+    for( size_t i = 0; i < depth.value(); ++i) {
+        if (i%2) feeder.add( new_value ); // pass lvalue
+        else     feeder.add( value_t(depth.value()-1) ); // pass rvalue
+    }
 }
 
 //! Standard form of the parallel_do functor object.
@@ -110,8 +119,9 @@ private:
     // Assert that parallel_do does not ever access body constructors
     TaskGeneratorBody () {}
     TaskGeneratorBody ( const TaskGeneratorBody& );
-    // TestBody() needs access to the default constructor
+    // These functions need access to the default constructor
     template<class Body, class Iterator> friend void TestBody( size_t );
+    template<class Body, class Iterator> friend void TestBody_MoveOnly( size_t );
 };
 
 /** Work item is passed by reference here. **/
@@ -159,9 +169,38 @@ public:
     }
 };
 
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+/** Work item is passed by rvalue reference here. **/
+class TaskGeneratorBody_RvalueRefVersion
+{
+public:
+    void operator() ( value_t&& depth, tbb::parallel_do_feeder<value_t>& feeder ) const {
+        do_work(depth, feeder);
+    }
+};
+#endif
 
 static value_t g_depths[N_DEPTHS] = {0, 1, 2, 3, 4, 0, 1, 0, 1, 2, 0, 1, 2, 3, 0, 1, 2, 0, 1, 2};
 
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+template<class Body, class Iterator>
+void TestBody_MoveIter ( const Body& body, Iterator begin, Iterator end  ) {
+    typedef std::move_iterator<Iterator> MoveIterator;
+    MoveIterator mbegin(begin);
+    MoveIterator mend(end);
+    g_tasks_observed = 0;
+    tbb::parallel_do(mbegin, mend, body);
+    ASSERT (g_tasks_observed == g_tasks_expected, NULL);
+}
+
+template<class Body, class Iterator>
+void TestBody_MoveOnly ( size_t depth ) {
+    typedef typename std::iterator_traits<Iterator>::value_type value_type;
+    value_type a_depths[N_DEPTHS] = {0, 1, 2, 3, 4, 0, 1, 0, 1, 2, 0, 1, 2, 3, 0, 1, 2, 0, 1, 2};
+    TestBody_MoveIter( Body(), Iterator(a_depths), Iterator(a_depths + depth));
+}
+#endif
+
 template<class Body, class Iterator>
 void TestBody ( size_t depth ) {
     typedef typename std::iterator_traits<Iterator>::value_type value_type;
@@ -172,10 +211,13 @@ void TestBody ( size_t depth ) {
     g_tasks_observed = 0;
     tbb::parallel_do(begin, end, body);
     ASSERT (g_tasks_observed == g_tasks_expected, NULL);
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+    TestBody_MoveIter( body, Iterator(a_depths), Iterator(a_depths + depth) );
+#endif
 }
 
 template<class Iterator>
-void TestIterator_RvalueOnly ( int /*nthread*/, size_t depth ) {
+void TestIterator_Common ( size_t depth ) {
     TestBody<FakeTaskGeneratorBody, Iterator> (depth);
     TestBody<FakeTaskGeneratorBody_ConstRefVersion, Iterator> (depth);
     TestBody<TaskGeneratorBody, Iterator> (depth);
@@ -184,30 +226,48 @@ void TestIterator_RvalueOnly ( int /*nthread*/, size_t depth ) {
 }
 
 template<class Iterator>
-void TestIterator ( int nthread, size_t depth ) {
-    TestIterator_RvalueOnly<Iterator>(nthread, depth);
+void TestIterator_Const ( size_t depth ) {
+    TestIterator_Common<Iterator>(depth);
+    TestBody<TaskGeneratorBody_ConstVolatileRefVersion, Iterator> (depth);
+}
+
+template<class Iterator>
+void TestIterator_Modifiable ( size_t depth ) {
+    TestIterator_Const<Iterator>(depth);
     TestBody<FakeTaskGeneratorBody_RefVersion, Iterator> (depth);
     TestBody<FakeTaskGeneratorBody_VolatileRefVersion, Iterator> (depth);
     TestBody<TaskGeneratorBody_RefVersion, Iterator> (depth);
     TestBody<TaskGeneratorBody_VolatileRefVersion, Iterator> (depth);
-    TestBody<TaskGeneratorBody_ConstVolatileRefVersion, Iterator> (depth);
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+    TestBody_MoveOnly<FakeTaskGeneratorBody_RvalueRefVersion, Iterator> (depth);
+    TestBody_MoveOnly<TaskGeneratorBody_RvalueRefVersion, Iterator> (depth);
+#endif
 }
 
-void Run( int nthread ) {
+template<class Iterator>
+void TestIterator_Movable ( size_t depth ) {
+    TestIterator_Common<Iterator>(depth);
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+    TestBody<FakeTaskGeneratorBody_RvalueRefVersion, Iterator> (depth);
+    TestBody<TaskGeneratorBody_RvalueRefVersion, Iterator> (depth);
+#endif
+}
+
+void Run( int /*nthread*/ ) {
     for( size_t depth = 0; depth <= N_DEPTHS; ++depth ) {
         g_tasks_expected = 0;
         for ( size_t i=0; i < depth; ++i )
             g_tasks_expected += FindNumOfTasks( g_depths[i].value() );
         // Test for iterators over values convertible to work item type
-        TestIterator_RvalueOnly<size_t*>(nthread, depth);
+        TestIterator_Movable<size_t*>(depth);
         // Test for random access iterators
-        TestIterator<value_t*>(nthread, depth);
+        TestIterator_Modifiable<value_t*>(depth);
         // Test for input iterators
-        TestIterator<Harness::InputIterator<value_t> >(nthread, depth);
+        TestIterator_Modifiable<Harness::InputIterator<value_t> >(depth);
         // Test for forward iterators
-        TestIterator<Harness::ForwardIterator<value_t> >(nthread, depth);
+        TestIterator_Modifiable<Harness::ForwardIterator<value_t> >(depth);
         // Test for const random access iterators
-        TestIterator_RvalueOnly<Harness::ConstRandomIterator<value_t> >(nthread, depth);
+        TestIterator_Const<Harness::ConstRandomIterator<value_t> >(depth);
     }
 }
 
@@ -225,6 +285,7 @@ struct set_to {
 
 #include "test_range_based_for.h"
 #include <functional>
+#include <deque>
 
 void range_do_test() {
     using namespace range_based_for_support_tests;
@@ -264,6 +325,83 @@ void range_do_test() {
     ASSERT(range_based_for_accumulate(v, std::plus<size_t>(), init_sum) == init_sum, "elements of v not all zeros");
 }
 
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+namespace TestMoveSem {
+    struct MovePreferable : Movable {
+        MovePreferable() : Movable(), addtofeed(true) {}
+        MovePreferable(bool addtofeed_) : Movable(), addtofeed(addtofeed_) {}
+        MovePreferable(MovePreferable&& other) : Movable(std::move(other)), addtofeed(other.addtofeed) {};
+        // base class is explicitly initialized in the copy ctor to avoid -Wextra warnings
+        MovePreferable(const MovePreferable& other) : Movable(other) { REPORT("Error: copy ctor preferred.\n"); };
+        MovePreferable& operator=(const MovePreferable&) { REPORT("Error: copy assign operator preferred.\n"); return *this; }
+        bool addtofeed;
+    };
+    struct MoveOnly : MovePreferable, NoCopy {
+        MoveOnly() : MovePreferable() {}
+        MoveOnly(bool addtofeed_) : MovePreferable(addtofeed_) {}
+        MoveOnly(MoveOnly&& other) : MovePreferable(std::move(other)) {};
+    };
+}
+
+template<typename T>
+void RecordAndAdd(const T& in, tbb::parallel_do_feeder<T>& feeder) {
+    ASSERT(in.alive, "Got dead object in body");
+    size_t i = ++g_tasks_observed;
+    if (in.addtofeed) {
+        if (i%2) feeder.add(T(false));
+        else {
+            T a(false);
+            feeder.add(std::move(a));
+        }
+    }
+}
+// Take an item by rvalue reference
+template<typename T>
+struct TestMoveIteratorBody {
+    void operator() (T&& in, tbb::parallel_do_feeder<T>& feeder) const { RecordAndAdd(in, feeder); }
+};
+// Take an item by value
+template<typename T>
+struct TestMoveIteratorBodyByValue {
+    void operator() (T in, tbb::parallel_do_feeder<T>& feeder) const { RecordAndAdd(in, feeder); }
+};
+
+template<typename Iterator, typename Body>
+void TestMoveIterator() {
+    typedef typename std::iterator_traits<Iterator>::value_type value_type;
+
+    Body body;
+    const size_t size = 65;
+    g_tasks_observed = 0;
+    value_type a[size];
+    tbb::parallel_do( std::make_move_iterator(Iterator(a)), std::make_move_iterator(Iterator(a+size)), body );
+    ASSERT(size * 2  == g_tasks_observed, NULL);
+}
+
+template<typename T>
+void DoTestMoveSemantics() {
+    TestMoveIterator< Harness::InputIterator<T>, TestMoveIteratorBody<T> >();
+    TestMoveIterator< Harness::ForwardIterator<T>, TestMoveIteratorBody<T> >();
+    TestMoveIterator< Harness::RandomIterator<T>, TestMoveIteratorBody<T> >();
+
+    TestMoveIterator< Harness::InputIterator<T>, TestMoveIteratorBodyByValue<T> >();
+    TestMoveIterator< Harness::ForwardIterator<T>, TestMoveIteratorBodyByValue<T> >();
+    TestMoveIterator< Harness::RandomIterator<T>, TestMoveIteratorBodyByValue<T> >();
+}
+
+void TestMoveSemantics() {
+    DoTestMoveSemantics<TestMoveSem::MovePreferable>();
+#if __TBB_CPP11_IS_COPY_CONSTRUCTIBLE_PRESENT && !__TBB_IS_COPY_CONSTRUCTIBLE_BROKEN
+    //  parallel_do uses is_copy_constructible to support non-copyable types
+    DoTestMoveSemantics<TestMoveSem::MoveOnly>();
+#endif
+}
+#else /* __TBB_CPP11_RVALUE_REF_PRESENT */
+void TestMoveSemantics() {
+    REPORT("Known issue: move support tests are skipped.\n");
+}
+#endif
+
 int TestMain () {
     if( MinThread<1 ) {
         REPORT("number of threads must be positive\n");
@@ -277,8 +415,10 @@ int TestMain () {
         // Test that all workers sleep when no work
         TestCPUUserTime(p);
     }
-    // This check must be performed after the scheduler terminated because only in this 
-    // case there is a guarantee that the workers already destroyed their last tasks. 
+    // This check must be performed after the scheduler terminated because only in this
+    // case there is a guarantee that the workers already destroyed their last tasks.
     ASSERT( g_values_counter == 0, "Value objects were leaked" );
+
+    TestMoveSemantics();
     return Harness::Done;
 }
