@@ -1,21 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "tbb/atomic.h"
@@ -27,6 +23,19 @@
 #include "harness_report.h"
 #include "harness_assert.h"
 
+bool CheckSignatures() {
+    // Checks that thread ids can be compared, in the way users would do it
+    THREAD::id id1, id2;
+    bool result = id1 == id2;
+    result |= id1 != id2;
+    result |= id1 < id2;
+    result |= id1 > id2;
+    result |= id1 <= id2;
+    result |= id1 >= id2;
+    tbb::tbb_hash<THREAD::id> hash;
+    return result |= hash(id1)==hash(id2);
+}
+
 static const int THRDS = 3;
 static const int THRDS_DETACH = 2;
 static tbb::atomic<int> sum;
@@ -41,9 +50,9 @@ class Base {
 protected:
     Base() : copy_throws(100) {++BaseCount;}
     Base( const Base& c ) : copy_throws(c.copy_throws) {
-        if( --copy_throws<=0 ) 
+        if( --copy_throws<=0 )
             __TBB_THROW(0);
-        ++BaseCount; 
+        ++BaseCount;
     }
     ~Base() {--BaseCount;}
 };
@@ -59,7 +68,6 @@ public:
     int value;
 };
 
-
 #include "harness_barrier.h"
 
 class ThreadFunc: Base {
@@ -72,7 +80,7 @@ public:
     void operator()(){
         real_ids[0] = THIS_THREAD::get_id();
         init_barrier.wait();
-        
+
         sum.fetch_and_add(1);
     }
     void operator()(int num){
@@ -81,7 +89,7 @@ public:
 
         sum.fetch_and_add(num);
     }
-    void operator()(int num, Data<0> dx) { 
+    void operator()(int num, Data<0> dx) {
         real_ids[num] = THIS_THREAD::get_id();
 
         const double WAIT = .1;
@@ -126,7 +134,7 @@ void CheckRelations( const THREAD::id ids[], int n, bool duplicates_allowed ) {
             for( int k=0; k<n; ++k ) {
                 const THREAD::id z = ids[j];
                 ASSERT( !(x<y && y<z) || x<z, "< is not transitive" );
-            }    
+            }
         }
     }
 }
@@ -140,13 +148,13 @@ public:
 };
 
 #if TBB_USE_EXCEPTIONS
-void CheckExceptionSafety() { 
+void CheckExceptionSafety() {
     int original_count = BaseCount;
-    // d loops over number of copies before throw occurs 
+    // d loops over number of copies before throw occurs
     for( int d=1; d<=3; ++d ) {
         // Try all combinations of throw/nothrow for f, x, and y's copy constructor.
         for( int i=0; i<8; ++i ) {
-            { 
+            {
                 const AnotherThreadFunc f = AnotherThreadFunc();
                 if( i&1 ) f.copy_throws = d;
                 const Data<1> x(0);
@@ -155,7 +163,7 @@ void CheckExceptionSafety() {
                 if( i&4 ) y.copy_throws = d;
                 bool exception_caught = false;
                 for( int j=0; j<3; ++j ) {
-                    try { 
+                    try {
                         switch(j) {
                             case 0: {THREAD t(f); t.join();} break;
                             case 1: {THREAD t(f,x); t.join();} break;
@@ -163,7 +171,7 @@ void CheckExceptionSafety() {
                         }
                     } catch(...) {
                         exception_caught = true;
-                    } 
+                    }
                     ASSERT( !exception_caught||(i&((1<<(j+1))-1))!=0, NULL );
                 }
             }
@@ -176,9 +184,8 @@ void CheckExceptionSafety() {
 #include <cstdio>
 
 #if __TBB_CPP11_RVALUE_REF_PRESENT
-
-tbb::tbb_thread returnThread() {
-    return tbb::tbb_thread();
+THREAD returnThread() {
+    return THREAD();
 }
 #endif
 
@@ -188,13 +195,13 @@ void RunTests() {
     Data<0> d100(100), d1(1), d0(0);
     const THREAD::id id_zero;
     THREAD::id id0, uniq_ids[THRDS];
-    
+
     THREAD thrs[THRDS];
     THREAD thr;
     THREAD thr0(t);
     THREAD thr1(t, 2);
     THREAD thr2(t, 1, d100);
-    
+
     ASSERT( thr0.get_id() != id_zero, NULL );
     id0 = thr0.get_id();
     tbb::move(thrs[0], thr0);
@@ -259,11 +266,11 @@ void RunTests() {
     ASSERT( BaseCount==4, "object leak detected" );
 
 #if TBB_USE_EXCEPTIONS
-    CheckExceptionSafety(); 
+    CheckExceptionSafety();
 #endif
 
     // Note: all tests involving BaseCount should be put before the tests
-    // involing detached threads, because there is no way of knowing when 
+    // involing detached threads, because there is no way of knowing when
     // a detached thread destroys its arguments.
 
     THREAD thr_detach_0(t, d0);
@@ -296,15 +303,3 @@ void RunTests() {
 
     ASSERT( THREAD::hardware_concurrency() > 0, NULL);
 }
-
-typedef bool (*id_relation)( THREAD::id, THREAD::id );
-
-id_relation CheckSignatures() {
-    id_relation r[6] = {&tbb::operator==,
-                        &tbb::operator!=,
-                        &tbb::operator<,
-                        &tbb::operator>,
-                        &tbb::operator<=,
-                        &tbb::operator>=};
-    return r[1];
-}
