@@ -1,4 +1,7 @@
 
+# defualt compiler unset
+define(COMPILER = "")
+
 # check whether user has Makevars file that might cause trouble
 makevars <- Sys.getenv("R_MAKEVARS_USER", unset = "~/.R/Makevars")
 if (file.exists(makevars)) {
@@ -24,11 +27,14 @@ if (file.exists(makevars)) {
 
 # Figure out the appropriate CXX prefix for the current
 # version of R + configuration.
-cxx <- NULL
+cxx <- "/usr/bin/c++"
 candidates <- c("CXX11", "CXX1X", "CXX")
 for (candidate in candidates) {
    value <- r_cmd_config(candidate)
    if (!is.null(value)) {
+      if (any(grepl("icpc", value))) {
+         define(COMPILER = "icc")
+      }
       cxx <- candidate
       break
    }
@@ -49,6 +55,8 @@ if (broken)
 # there as opposed to the bundled version
 cppflags <- read_r_config("CPPFLAGS", envir = NULL)[[1]]
 cppflags <- sub("(?: )?-I/usr/local/include", "", cppflags)
+cppflags <- sub("(?: )?-I/opt/homebrew/include", "", cppflags)
+cppflags <- sub("(?: )?-I/opt/local/libexec/onetbb/include", "", cppflags)
 
 # define the set of flags appropriate to the current
 # configuration of R
@@ -115,7 +123,6 @@ if (getRversion() < "4.0") {
 }
 
 # on Solaris, check if we're using gcc or g++
-define(COMPILER = "")
 if (Sys.info()[["sysname"]] == "SunOS") {
    cxx <- r_cmd_config("CXX")
    version <- system(paste(cxx, "--version"), intern = TRUE)
@@ -126,3 +133,81 @@ if (Sys.info()[["sysname"]] == "SunOS") {
    }
 }
 
+# try and figure out path to TBB
+tbbRoot <- Sys.getenv("TBB_ROOT", unset = NA)
+tbbLib  <- Sys.getenv("TBB_LIB", unset = NA)
+tbbInc  <- Sys.getenv("TBB_INC", unset = NA)
+
+# check TBB_ROOT first if defined
+if (!is.na(tbbRoot)) {
+   
+   if (is.na(tbbLib)) {
+      tbbLib <- file.path(tbbRoot, "lib")
+   }
+   
+   if (is.na(tbbInc)) {
+      tbbInc <- file.path(tbbRoot, "include")
+   }
+   
+}
+
+# if TBB_LIB is defined, guess TBB_INC
+if (!is.na(tbbLib) && is.na(tbbInc)) {
+   tbbIncCandidate <- file.path(tbbLib, "../include")
+   if (file.exists(tbbIncCandidate)) {
+      tbbInc <- normalizePath(tbbIncCandidate)
+   }
+}
+
+# if TBB_LIB and TBB_INC are still not defined, try auto-detecting
+tryAutoDetect <-
+   .Platform$OS.type == "unix" &&
+   Sys.getenv("TBB_AUTODETECT", unset = "FALSE") == "TRUE" &&
+   is.na(tbbLib) &&
+   is.na(tbbInc)
+
+if (tryAutoDetect) {
+   
+   sysInfo <- as.list(Sys.info())
+   
+   homebrewPrefix <- if (sysInfo$sysname == "Darwin") {
+      "/opt/homebrew"
+   } else {
+      "/usr/local"
+   }
+   
+   tbbLibSearch <- if (sysInfo$sysname == "Darwin") {
+      file.path(homebrewPrefix, "opt/tbb/lib/libtbb.dylib")
+   } else {
+      Sys.glob(c(
+         "/usr/*/libtbb.so",
+         "/usr/*/*/libtbb.so",
+         "/usr/*/*/*/libtbb.so"
+      ))
+   }
+   
+   tbbIncSearch <- if (sysInfo$sysname == "Darwin") {
+      file.path(homebrewPrefix, "opt/tbb/include/tbb")
+   } else {
+      Sys.glob(c(
+         "/usr/include/tbb.h",
+         "/usr/include/*/tbb.h"
+      ))
+   }
+   
+   if (length(tbbLibSearch) &&
+       length(tbbIncSearch) &&
+       file.exists(tbbLibSearch[[1L]]) &&
+       file.exists(tbbIncSearch[[1L]]))
+   {
+      tbbLib <- dirname(tbbLibSearch[[1L]])
+      tbbInc <- dirname(tbbIncSearch[[1L]])
+   }
+   
+}
+
+# now, define TBB_LIB and TBB_INC as appropriate
+define(
+   TBB_LIB = if (!is.na(tbbLib)) tbbLib else "",
+   TBB_INC = if (!is.na(tbbInc)) tbbInc else ""
+)
